import requests
import os
from pprint import pprint

_memory = {}
def set_variable(key: str, val: str):
    '''设置内存变量'''

    _memory[key] = val

def get_data_fromlocal(ip,port,function,key): 
    '''这个函数通过本地egolocal进程来跟远端通信（在不信任的环境下）'''

    resp = requests.post("http://{0}:{1}/{2}/{3}".format(ip,port,function,key)).json()
    if resp["errorflag"]:
        raise RuntimeError(resp["errormsg"])
    val = resp["val"]

    return val

def get_data_fromenv(server_ip,password,type,key):
    '''这个函数通过环境变量来跟远端通信（在信任的环境下）'''

    resp = requests.post("http://{0}/get/{1}/{2}".format(server_ip, type, key), json = {"password": password}).json()
    if resp["errorflag"]:
        raise RuntimeError(resp["errormsg"])
    val = resp["val"]
    return val

def get_data(port, local_ip, server_ip, password , type, key):

    # 如果内存里已经设置了值，就用内存中的值。
    _from_memo = _memory.get(key) 
    if _from_memo is not None:   
        return _from_memo

    server_ip = server_ip or os.environ.get("EGOROV_SERVER")
    password = password or os.environ.get("EGOROV_PWD")
    if server_ip and password: # 如果传入了server_ip和密码，或者这两个能从环境变量里找到，那就直接用这个跟远端通信。
        return get_data_fromenv(server_ip, password, type, key)

    # 跟本地进程通信
    function = "ask_{0}".format(type)
    return get_data_fromlocal(local_ip, port, function, key)



def get_variable(key, port = 11451, local_ip = "127.0.0.1", server_ip = None,password = None):
    return get_data(port, local_ip, server_ip, password , "variable", key)

def get_function(key, port = 11451, local_ip = "127.0.0.1", server_ip = None,password = None):
    return get_data(port, local_ip, server_ip, password , "function", key)


class _Egorov:
    def __init__(self):
        pass
    def __getitem__(self, key: str):
        try:
            return get_variable(key)
        except RuntimeError:
            return get_function(key)
Egorov = _Egorov()


from pathlib import Path
import os

template = """server {
    listen 80;  
    server_name %s; 

    location / {
        proxy_pass %s;
        proxy_set_header Host $host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
        proxy_set_header X-Forwarded-Proto $scheme;
    }
}
"""

def run():
    server_name = input("Please input the server name: ").strip()
    url         = input("Please input the target url: ").strip()
    nginx       = input("Please input nginx location (enter for default): ").strip()
    my_name     = "egorovsystem"

    content = template % (server_name, url)

    nginx = Path("/etc/nginx") if nginx == "" else Path(nginx)

    with open(nginx / "sites-available" / my_name, "w") as f:
        f.write(content)
    
    os.system(f"rm {nginx / 'sites-enabled' / my_name}")
    os.system(f"ln -s {nginx / 'sites-available' / my_name} {nginx / 'sites-enabled' / my_name}")
    os.system("sudo systemctl restart nginx")


if __name__ == "__main__":
    run()


import json

def JSONDecode(s):
    s = s.strip()
    if s == "":
        return {}
    
    ret = {}
    try:
        ret = json.loads(s)
    except json.JSONDecodeError:
        pass

    return ret



from django.urls import path
from .views import get_variable , get_function, add_password, delete_password
from functools import partial

urlpatterns = [

    path("get/variable/<str:key>"       , get_variable) , 
    path("get/function/<str:key>"       , get_function) , 
    path("add_password/<str:key>"       , add_password) , 
    path("delete_password/<str:key>"    , delete_password) , 
]


# Generated by Django 4.1.5 on 2023-07-13 08:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('server', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='function',
            name='type',
            field=models.TextField(blank=True, default='cmd', null=True),
        ),
    ]



from setuptools import setup, find_packages

with open("README.md", encoding="utf-8") as f:
	readme = f.read()

with open("LICENSE", encoding="utf-8") as f:
	license = f.read()

# pip 在搞什么飞机
try:
	with open("requirements.txt", encoding="utf-8") as f:
		reqs = f.read()
except:
	reqs = ""

print (find_packages())
pkgs = [p for p in find_packages() if p.startswith("ego")]
print(pkgs)

setup(
	name					= "egorovsystem",
	version					= "0.0.9.post13",
	description				= "",
	long_description		= readme,
	long_description_content_type	= "text/markdown",
	license					= license,
	author					= "Yongyi Yang",
	author_email 			= "yongyi@umich.edu",
	python_requires			= ">=3.6",
	packages				= pkgs,
	install_requires		= reqs.strip().split("\n"),
	entry_points			= {"console_scripts": [
		"egoserver-init=egoserver.run:init" , 
		"egoserver-run=egoserver.run:run" , 
		"egolocal-run=egolocal.main:run" , 
		"EGO=egorovsystem.run_command:run" , 
		"egoserver-setnginx=egoserver_setnginx.main:run" , 
	]} ,
)



from django.contrib import admin
from .models import Variable , Function, Password

class VariableAdmin(admin.ModelAdmin):
    pass
class FunctionAdmin(admin.ModelAdmin):
    pass
class PasswordAdmin(admin.ModelAdmin):
    pass

# Register your models here.
admin.site.register(Variable     , VariableAdmin  )
admin.site.register(Function     , FunctionAdmin  ) 
admin.site.register(Password     , PasswordAdmin  ) 
